@@ -2,6 +2,7 @@
 #![feature(never_type)]
 #![feature(try_trait_v2)]
 #![feature(control_flow_enum)]
+#![allow(clippy::type_complexity)]
 #![allow(deprecated)]
 
 #[cfg(feature = "derive")]
@@ -9,14 +10,13 @@ pub use elrond_wasm_derive::{self as derive, contract, module, proxy};
 
 // re-export basic heap types
 extern crate alloc;
-pub use alloc::boxed::Box;
-pub use alloc::string::String;
-pub use alloc::vec::Vec;
+pub use alloc::{boxed::Box, string::String, vec::Vec};
 
 pub use elrond_codec;
 
 pub mod abi;
 pub mod api;
+pub mod contract_base;
 pub mod err_msg;
 pub mod esdt;
 pub mod hex_call_data;
