@@ -1,24 +1,19 @@
 mod linked_list_mapper;
 mod map_mapper;
-mod map_mapper_safe;
 mod map_storage_mapper;
-mod map_storage_mapper_safe;
 mod mapper;
 mod set_mapper;
-mod set_mapper_safe;
 mod single_value_mapper;
+mod token_attributes_mapper;
 mod user_mapper;
 mod vec_mapper;
 
 pub use linked_list_mapper::LinkedListMapper;
 pub use map_mapper::MapMapper;
-pub use map_mapper_safe::SafeMapMapper;
 pub use map_storage_mapper::MapStorageMapper;
-pub use map_storage_mapper_safe::SafeMapStorageMapper;
-pub use mapper::StorageClearable;
-pub use mapper::StorageMapper;
+pub use mapper::{StorageClearable, StorageMapper};
 pub use set_mapper::SetMapper;
-pub use set_mapper_safe::SafeSetMapper;
 pub use single_value_mapper::SingleValueMapper;
+pub use token_attributes_mapper::TokenAttributesMapper;
 pub use user_mapper::UserMapper;
 pub use vec_mapper::VecMapper;
