@@ -1,9 +1,11 @@
 use super::{StorageClearable, StorageMapper};
-use crate::abi::{TypeAbi, TypeDescriptionContainer, TypeName};
-use crate::api::{EndpointFinishApi, ErrorApi, StorageReadApi, StorageWriteApi};
-use crate::io::EndpointResult;
-use crate::storage::{storage_get, storage_set};
-use crate::types::{BoxedBytes, MultiResultVec};
+use crate::{
+    abi::{TypeAbi, TypeDescriptionContainer, TypeName},
+    api::{EndpointFinishApi, ErrorApi, ManagedTypeApi, StorageReadApi, StorageWriteApi},
+    io::EndpointResult,
+    storage::{storage_clear, storage_get, storage_get_len, storage_set, StorageKey},
+    types::MultiResultVec,
+};
 use alloc::vec::Vec;
 use core::{marker::PhantomData, usize};
 use elrond_codec::{TopDecode, TopEncode};
@@ -11,13 +13,6 @@ use elrond_codec::{TopDecode, TopEncode};
 const ITEM_SUFFIX: &[u8] = b".item";
 const LEN_SUFFIX: &[u8] = b".len";
 
-fn compute_item_key(prefix: &[u8], index: usize) -> BoxedBytes {
-    // cast to u32, so it also works correctly in debug mode on x64 architectures
-    // would be nice to go via the framework serialization, but it currently has a little overhead over this
-    let index_bytes = (index as u32).to_be_bytes();
-    BoxedBytes::from_concat(&[prefix, ITEM_SUFFIX, &index_bytes[..]])
-}
-
 /// Manages a list of items of the same type.
 /// Saves each of the items under a separate key in storage.
 /// To produce each individual key, it concatenates the main key with a serialized 4-byte index.
@@ -26,23 +21,28 @@ fn compute_item_key(prefix: &[u8], index: usize) -> BoxedBytes {
 /// The count is always kept in sync automatically.
 pub struct VecMapper<SA, T>
 where
-    SA: StorageReadApi + StorageWriteApi + ErrorApi + Clone + 'static,
+    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
     T: TopEncode + TopDecode + 'static,
 {
     api: SA,
-    main_key: BoxedBytes,
+    base_key: StorageKey<SA>,
+    len_key: StorageKey<SA>,
     _phantom: core::marker::PhantomData<T>,
 }
 
 impl<SA, T> StorageMapper<SA> for VecMapper<SA, T>
 where
-    SA: StorageReadApi + StorageWriteApi + ErrorApi + Clone + 'static,
+    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
     T: TopEncode + TopDecode,
 {
-    fn new(api: SA, main_key: BoxedBytes) -> Self {
+    fn new(api: SA, base_key: StorageKey<SA>) -> Self {
+        let mut len_key = base_key.clone();
+        len_key.append_bytes(LEN_SUFFIX);
+
         VecMapper {
             api,
-            main_key,
+            base_key,
+            len_key,
             _phantom: PhantomData,
         }
     }
@@ -50,7 +50,7 @@ where
 
 impl<SA, T> StorageClearable for VecMapper<SA, T>
 where
-    SA: StorageReadApi + StorageWriteApi + ErrorApi + Clone + 'static,
+    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
     T: TopEncode + TopDecode,
 {
     fn clear(&mut self) {
@@ -60,24 +60,23 @@ where
 
 impl<SA, T> VecMapper<SA, T>
 where
-    SA: StorageReadApi + StorageWriteApi + ErrorApi + Clone + 'static,
+    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
     T: TopEncode + TopDecode,
 {
-    fn item_key(&self, index: usize) -> BoxedBytes {
-        compute_item_key(self.main_key.as_slice(), index)
-    }
-
-    fn len_key(&self) -> BoxedBytes {
-        BoxedBytes::from_concat(&[self.main_key.as_slice(), LEN_SUFFIX])
+    fn item_key(&self, index: usize) -> StorageKey<SA> {
+        let mut item_key = self.base_key.clone();
+        item_key.append_bytes(ITEM_SUFFIX);
+        item_key.append_item(&index);
+        item_key
     }
 
     fn save_count(&self, new_len: usize) {
-        storage_set(self.api.clone(), self.len_key().as_slice(), &new_len);
+        storage_set(self.api.clone(), &self.len_key, &new_len);
     }
 
     /// Number of items managed by the mapper.
     pub fn len(&self) -> usize {
-        storage_get(self.api.clone(), self.len_key().as_slice())
+        storage_get(self.api.clone(), &self.len_key)
     }
 
     /// True if no items present in the mapper.
@@ -90,7 +89,7 @@ where
     pub fn push(&mut self, item: &T) -> usize {
         let mut len = self.len();
         len += 1;
-        storage_set(self.api.clone(), self.item_key(len).as_slice(), item);
+        storage_set(self.api.clone(), &self.item_key(len), item);
         self.save_count(len);
         len
     }
@@ -102,7 +101,7 @@ where
         let mut len = self.len();
         for item in items {
             len += 1;
-            storage_set(self.api.clone(), self.item_key(len).as_slice(), item);
+            storage_set(self.api.clone(), &self.item_key(len), item);
         }
         self.save_count(len);
         len
@@ -121,7 +120,7 @@ where
     /// There are no restrictions on the index,
     /// calling for an invalid index will simply return the zero-value.
     pub fn get_unchecked(&self, index: usize) -> T {
-        storage_get(self.api.clone(), self.item_key(index).as_slice())
+        storage_get(self.api.clone(), &self.item_key(index))
     }
 
     /// Get item at index from storage.
@@ -140,7 +139,7 @@ where
     /// There are no restrictions on the index,
     /// calling for an invalid index will simply return `true`.
     pub fn item_is_empty_unchecked(&self, index: usize) -> bool {
-        self.api.storage_load_len(self.item_key(index).as_slice()) == 0
+        storage_get_len(self.api.clone(), &self.item_key(index)) == 0
     }
 
     /// Checks whether or not there is anything ins storage at index.
@@ -163,7 +162,7 @@ where
 
     /// Keeping `set_unchecked` private on purpose, so developers don't write out of index limits by accident.
     fn set_unchecked(&self, index: usize, item: &T) {
-        storage_set(self.api.clone(), self.item_key(index).as_slice(), item);
+        storage_set(self.api.clone(), &self.item_key(index), item);
     }
 
     /// Clears item at index from storage.
@@ -179,8 +178,7 @@ where
     /// There are no restrictions on the index,
     /// calling for an invalid index will simply do nothing.
     pub fn clear_entry_unchecked(&self, index: usize) {
-        self.api
-            .storage_store_slice_u8(self.item_key(index).as_slice(), &[]);
+        storage_clear(self.api.clone(), &self.item_key(index));
     }
 
     /// Loads all items from storage and places them in a Vec.
@@ -199,8 +197,7 @@ where
     pub fn clear(&mut self) {
         let len = self.len();
         for i in 1..=len {
-            self.api
-                .storage_store_slice_u8(self.item_key(i).as_slice(), &[]);
+            storage_clear(self.api.clone(), &self.item_key(i));
         }
         self.save_count(0);
     }
@@ -209,14 +206,14 @@ where
 /// Behaves like a MultiResultVec when an endpoint result.
 impl<SA, T> EndpointResult for VecMapper<SA, T>
 where
-    SA: StorageReadApi + StorageWriteApi + ErrorApi + Clone + 'static,
+    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
     T: TopEncode + TopDecode + EndpointResult,
 {
     type DecodeAs = MultiResultVec<T::DecodeAs>;
 
     fn finish<FA>(&self, api: FA)
     where
-        FA: EndpointFinishApi + Clone + 'static,
+        FA: ManagedTypeApi + EndpointFinishApi + Clone + 'static,
     {
         let v = self.load_as_vec();
         MultiResultVec::<T>::from(v).finish(api);
@@ -226,7 +223,7 @@ where
 /// Behaves like a MultiResultVec when an endpoint result.
 impl<SA, T> TypeAbi for VecMapper<SA, T>
 where
-    SA: StorageReadApi + StorageWriteApi + ErrorApi + Clone + 'static,
+    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
     T: TopEncode + TopDecode + TypeAbi,
 {
     fn type_name() -> TypeName {
