@@ -4,16 +4,18 @@ mod callback_call;
 mod callback_selector_result;
 mod contract_call;
 mod contract_deploy;
+mod managed_arg_buffer;
 mod send_egld;
 mod send_esdt;
 mod send_token;
 
 pub use arg_buffer::ArgBuffer;
 pub use async_call::AsyncCall;
-pub use callback_call::CallbackCall;
+pub use callback_call::{new_callback_call, CallbackCall};
 pub use callback_selector_result::CallbackSelectorResult;
 pub use contract_call::{new_contract_call, ContractCall};
 pub use contract_deploy::{new_contract_deploy, ContractDeploy};
+pub use managed_arg_buffer::ManagedArgBuffer;
 pub use send_egld::SendEgld;
 pub use send_esdt::SendEsdt;
 pub use send_token::SendToken;
